@@ -150,14 +150,14 @@ class Configure( object ):
             return
         self._record = False
         self.verbose = Configure.OUT_QUIET
-        file = self.open( 'log/config.info.txt', 'w' )
+        log_info_file = self.open( 'log/config.info.txt', 'w' )
         for line in self._log_info:
-            file.write( line )
-        file.close()
-        file = self.open( 'log/config.verbose.txt', 'w' )
+            log_info_file.write( line )
+        log_info_file.close()
+        log_verbose_file = self.open( 'log/config.verbose.txt', 'w' )
         for line in self._log_verbose:
-            file.write( line )
-        file.close()
+            log_verbose_file.write( line )
+        log_verbose_file.close()
 
     ## Find executable by searching path.
     ## On success, returns full pathname of executable.
@@ -168,11 +168,7 @@ class Configure( object ):
                 return name
             return None
 
-        if not os.environ.has_key( 'PATH' ) or os.environ[ 'PATH' ] == '':
-            path = os.defpath
-        else:
-            path = os.environ['PATH']
-
+        path = os.getenv( 'PATH' ) or os.defpath
         for dir in path.split( os.pathsep ):
             f = os.path.join( dir, name )
             if os.access( f, os.X_OK ):
@@ -340,9 +336,8 @@ class CCProbe( Action ):
 
     def _action( self ):
         ## write program file
-        file = open( 'conftest.c', 'w' )
-        file.write( self.test_file )
-        file.close()
+        with open( 'conftest.c', 'w' ) as out_file:
+            out_file.write( self.test_file )
         ## pipe and redirect stderr to stdout; effects communicate result
         pipe = subprocess.Popen( '%s -c -o conftest.o conftest.c' % self.command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT )
 
@@ -382,9 +377,8 @@ class LDProbe( Action ):
 
     def _action( self ):
         ## write program file
-        file = open( 'conftest.c', 'w' )
-        file.write( self.test_file )
-        file.close()
+        with open( 'conftest.c', 'w' ) as out_file:
+            out_file.write( self.test_file )
         ## pipe and redirect stderr to stdout; effects communicate result
         pipe = subprocess.Popen( '%s -o conftest conftest.c %s' % (self.command, self.lib), shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT )
 
@@ -429,10 +423,7 @@ class HostTupleProbe( ShellProbe, list ):
         super( HostTupleProbe, self ).__init__( 'host tuple', '%s/config.guess' % (cfg.dir), abort=True, head=True )
 
     def _parseSession( self ):
-        if len(self.session):
-            self.spec = self.session[0]
-        else:
-            self.spec = ''
+        self.spec = self.session[0] if self.session else ''
 
         ## grok GNU host tuples
         m = re.match( HostTupleProbe.GNU_TUPLE_RE, self.spec )
@@ -503,7 +494,7 @@ class BuildAction( Action, list ):
             if self.match( '*mingw*' ):
                 self.systemf = 'MinGW'
             elif self.systemf:
-                self.systemf[0] = self.systemf[0].upper()
+                self.systemf = self.systemf.capitalize()
             self.title = '%s %s' % (build.systemf,self.machine)
         else:
             self.title = '%s %s' % (build.systemf,arch.mode.mode)
@@ -590,6 +581,9 @@ class ArchAction( Action ):
             pass
         elif host.match( '*-*-solaris*' ):
             pass
+        elif host.match( '*-*-freebsd.*' ):
+            self.mode['i386']   = 'i386-portsbuild-freebsd%s' % (host.release)
+            self.mode['amd64'] = 'amd64-portsbuild-freebsd%s' % (host.release)
         else:
             self.msg_pass = 'WARNING'
 
@@ -713,7 +707,7 @@ class RepoProbe( ShellProbe ):
         self.rev       = 0
         self.hash      = 'deadbeaf'
         self.shorthash = 'deadbea'
-        self.date      = datetime(1, 1, 1)
+        self.date      = None
         self.official  = 0
         self.type      = 'developer'
 
@@ -757,11 +751,10 @@ class RepoProbe( ShellProbe ):
                 self.shorthash = value[:7]
 
         # type-classification via repository URL
-        official_url = 'https://github.com/HandBrake/HandBrake.git' # HTTPS
-        if self.url == 'git@github.com:HandBrake/HandBrake.git':    # SSH
-            self.url = official_url
+        if self.url == project.url_repo_ssh:
+            self.url = project.url_repo  # official repo, SSH to HTTPS
 
-        if self.url == official_url:
+        if self.url == project.url_repo:
             self.official = 1
             if not options.snapshot and self.hash == self.tag_hash:
                 self.type = 'release'
@@ -781,12 +774,11 @@ class RepoProbe( ShellProbe ):
         try:
             hvp = os.path.join( cfg.src_dir, 'version.txt' )
             if os.path.isfile( hvp ) and os.path.getsize( hvp ) > 0:
-                file = open( hvp, 'r' )
-                self.session = file.readlines()
-                file.close()
+                with open( hvp, 'r' ) as in_file:
+                    self.session = in_file.readlines()
                 if self.session:
                     self._parseSession()
-            if self.rev != 0:
+            if self.hash is not empty and self.hash is not 'deadbeaf':
                 cfg.infof( '(pass)\n' )
             else:
                 cfg.infof( '(fail)\n' )
@@ -809,6 +801,8 @@ class Project( Action ):
         self.acro_lower    = 'hb'
         self.acro_upper    = 'HB'
         self.url_website   = 'https://handbrake.fr'
+        self.url_repo      = 'https://github.com/HandBrake/HandBrake.git'
+        self.url_repo_ssh  = 'git@github.com:HandBrake/HandBrake.git'
         self.url_community = 'https://forum.handbrake.fr'
         self.url_irc       = 'irc://irc.freenode.net/handbrake'
 
@@ -829,6 +823,10 @@ class Project( Action ):
         else:
             url_arch = ''
 
+        if repo.date is None:
+            cfg.errln( '%s is missing version information it needs to build properly.\nClone the official git repository at %s\nor download an official source archive from %s\n', self.name, self.url_repo, self.url_website )
+            sys.exit( 1 )
+
         if repo.tag != '':
             m = re.match( '^([0-9]+)\.([0-9]+)\.([0-9]+)-?(.+)?$', repo.tag )
             if not m:
@@ -969,12 +967,7 @@ class VersionProbe( Action ):
         ## read data into memory buffers
         data = pipe.communicate()
         self.fail = pipe.returncode != 0
-
-        if data[0]:
-            self.session = data[0].splitlines()
-        else:
-            self.session = []
-
+        self.session = data[0].splitlines() if data[0] else []
         self.svers = '0.0.0'
         self.ivers = [0,0,0]
 
@@ -1091,22 +1084,22 @@ class ConfigDocument:
     def __init__( self ):
         self._elements = []
 
-    def _outputMake( self, file, namelen, name, value, append ):
+    def _outputMake( self, out_file, namelen, name, value, append ):
         if append:
             if value == None or len(str(value)) == 0:
-                file.write( '%-*s +=\n' % (namelen, name) )
+                out_file.write( '%-*s +=\n' % (namelen, name) )
             else:
-                file.write( '%-*s += %s\n' % (namelen, name, value) )
+                out_file.write( '%-*s += %s\n' % (namelen, name, value) )
         else:
             if value == None or len(str(value)) == 0:
-                file.write( '%-*s  =\n' % (namelen, name) )
+                out_file.write( '%-*s  =\n' % (namelen, name) )
             else:
-                file.write( '%-*s  = %s\n' % (namelen, name, value) )
+                out_file.write( '%-*s  = %s\n' % (namelen, name, value) )
 
-    def _outputM4( self, file, namelen, name, value ):
+    def _outputM4( self, out_file, namelen, name, value ):
         namelen += 7
         name = '<<__%s>>,' % name.replace( '.', '_' )
-        file.write( 'define(%-*s  <<%s>>)dnl\n' % (namelen, name, value ))
+        out_file.write( 'define(%-*s  <<%s>>)dnl\n' % (namelen, name, value ))
 
     def add( self, name, value, append=False ):
         self._elements.append( [name,value,append] )
@@ -1124,7 +1117,7 @@ class ConfigDocument:
     def addM4( self, line ):
         self._elements.append( ('?m4',line) )
 
-    def output( self, file, type ):
+    def output( self, out_file, type ):
         namelen = 0
         for item in self._elements:
             if item == None or item[0].find( '?' ) == 0:
@@ -1134,19 +1127,19 @@ class ConfigDocument:
         for item in self._elements:
             if item == None:
                 if type == 'm4':
-                    file.write( 'dnl\n' )
+                    out_file.write( 'dnl\n' )
                 else:
-                    file.write( '\n' )
+                    out_file.write( '\n' )
                 continue
             if item[0].find( '?' ) == 0:
                 if item[0].find( type, 1 ) == 1:
-                    file.write( '%s\n' % (item[1]) )
+                    out_file.write( '%s\n' % (item[1]) )
                 continue
 
             if type == 'm4':
-                self._outputM4( file, namelen, item[0], item[1] )
+                self._outputM4( out_file, namelen, item[0], item[1] )
             else:
-                self._outputMake( file, namelen, item[0], item[1], item[2] )
+                self._outputMake( out_file, namelen, item[0], item[1], item[2] )
 
     def update( self, name, value ):
         for item in self._elements:
@@ -1168,11 +1161,11 @@ class ConfigDocument:
         ftmp = cfg.mktmpname(fname)
         try:
             try:
-                file = cfg.open( ftmp, 'w' )
-                self.output( file, type )
+                out_file = cfg.open( ftmp, 'w' )
+                self.output( out_file, type )
             finally:
                 try:
-                    file.close()
+                    out_file.close()
                 except:
                     pass
         except Exception, x:
@@ -1202,12 +1195,12 @@ def encodeDistfileConfig():
     }
     try:
         try:
-            file = cfg.open( ftmp, 'w' )
-            json.dump(data, file)
-            file.write('\n')
+            out_file = cfg.open( ftmp, 'w' )
+            json.dump(data, out_file)
+            out_file.write('\n')
         finally:
             try:
-                file.close()
+                out_file.close()
             except:
                 pass
     except Exception, x:
@@ -1294,27 +1287,32 @@ def createCLI():
     h = IfHost( 'enable assembly code in non-contrib modules', 'NOMATCH*-*-darwin*', 'NOMATCH*-*-linux*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-asm', default=False, action='store_true', help=h )
 
-    h = IfHost( 'disable GTK GUI', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'disable GTK GUI', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--disable-gtk', default=False, action='store_true', help=h )
-    h = IfHost( 'disable GTK GUI update checks', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+
+    h = IfHost( 'disable GTK GUI update checks', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
+
     grp.add_option( '--disable-gtk-update-checks', default=False, action='store_true', help=h )
+
     h = IfHost( 'enable GTK GUI (mingw)', '*-*-mingw*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-gtk-mingw', default=False, action='store_true', help=h )
-    h = IfHost( 'disable gstreamer (live preview)', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+
+    h = IfHost( 'disable GStreamer (live preview)', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
+
     grp.add_option( '--disable-gst', default=False, action='store_true', help=h )
-    h = IfHost( 'enable use of Intel Quick Sync Video hardware acceleration', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+
+    h = IfHost( 'enable Intel Quick Sync Video (QSV) hardware acceleration. (Windows and Linux only)', '*-*-linux*', '*-*-mingw*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-qsv', default=False, action='store_true', help=h )
 
-
-    h = IfHost( 'enable use of x265 encoding', '*-*-*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--enable-x265', default=True, action='store_true', help=h )
+    h = IfHost( 'enable x265 video encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-x265', dest="enable_x265", default=True, action='store_true', help=h )
     grp.add_option( '--disable-x265', dest="enable_x265", action='store_false' )
 
-    h = IfHost( 'enable use of fdk-aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'enable FDK AAC audio encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-fdk-aac', dest="enable_fdk_aac", default=False, action='store_true', help=h )
     grp.add_option( '--disable-fdk-aac', dest="enable_fdk_aac", action='store_false' )
 
-    h = IfHost( 'enable use of libav aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'enable Libav AAC audio encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-libav-aac', dest="enable_libav_aac", default=not host.match( '*-*-darwin*' ), action='store_true', help=h )
     grp.add_option( '--disable-libav-aac', dest="enable_libav_aac", action='store_false' )
 
@@ -1351,12 +1349,8 @@ def createCLI():
     grp.add_option( '--enable-local-cmake', default=False, action='store_true', help=h )
     h = IfHost( 'Build and use local pkg-config', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-local-pkgconfig', default=False, action='store_true', help=h )
-
-    h = IfHost( 'Build extra contribs for flatpak packaging', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--flatpak', default=False, action='store_true', help=h )
     cli.add_option_group( grp )
 
-
     ## add Xcode options
     if host.match( '*-*-darwin*' ):
         grp = OptionGroup( cli, 'Xcode Options' )
@@ -1385,6 +1379,9 @@ def createCLI():
     grp = OptionGroup( cli, 'Build Options' )
     grp.add_option( '--snapshot', default=False, action='store_true',
                     help='Force a snapshot build' )
+
+    h = IfHost( 'Build extra contribs for flatpak packaging', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--flatpak', default=False, action='store_true', help=h )
     cli.add_option_group( grp )
 
     return cli
@@ -1531,6 +1528,7 @@ try:
         ranlib   = ToolProbe( 'RANLIB.exe',   'ranlib' )
         strip    = ToolProbe( 'STRIP.exe',    'strip' )
         tar      = ToolProbe( 'TAR.exe',      'gtar', 'tar' )
+        nasm     = ToolProbe( 'NASM.exe',     'nasm', abort=False, minversion=[2,13,0] )
         yasm     = ToolProbe( 'YASM.exe',     'yasm', abort=False, minversion=[1,2,0] )
         autoconf = ToolProbe( 'AUTOCONF.exe', 'autoconf', abort=False )
         automake = ToolProbe( 'AUTOMAKE.exe', 'automake', abort=False )
@@ -1591,6 +1589,13 @@ try:
     for action in Action.actions:
         action.run()
 
+    ## fail on missing or old nasm where needed
+    if host.match( '*-*-darwin*' ) or options.cross:
+        if Tools.nasm.fail:
+            raise AbortError( 'error: nasm missing\n' )
+        elif Tools.nasm.version.inadequate():
+            raise AbortError( 'error: minimum required nasm version is %s and %s is %s\n' % ('.'.join([str(i) for i in Tools.nasm.version.minversion]),Tools.nasm.pathname,Tools.nasm.version.svers) )
+
     ## enable local yasm when yasm probe fails or version is too old
     ## x264 requires 1.2.0+
     if not options.enable_local_yasm:
@@ -1859,7 +1864,7 @@ int main()
     doc.add( 'FEATURE.gtk.mingw',  int( options.enable_gtk_mingw ))
     doc.add( 'FEATURE.gst',        int( not options.disable_gst ))
     doc.add( 'FEATURE.fdk_aac',    int( options.enable_fdk_aac ))
-    doc.add( 'FEATURE.libav_aac',  int( options.enable_libav_aac ))
+    doc.add( 'FEATURE.libav_aac',  int( options.enable_libav_aac or build.system == 'mingw' ))
     doc.add( 'FEATURE.qsv',        int( options.enable_qsv ))
     doc.add( 'FEATURE.xcode',      int( not (Tools.xcodebuild.fail or options.disable_xcode or options.cross) ))
     doc.add( 'FEATURE.x265',       int( options.enable_x265 ))
@@ -1893,8 +1898,11 @@ int main()
             doc.add( 'HAS.regex', 1 )
         if strtok_r.fail:
             doc.add( 'COMPAT.strtok_r', 1 )
+
     else:
         doc.addBlank()
+        if build.system == 'freebsd':
+            doc.add( 'HAS.pthread', 1 )
         if not strerror_r.fail:
             doc.add( 'HAS.strerror_r', 1 )
 
@@ -1928,6 +1936,8 @@ int main()
         doc.add( 'LIBHB.GCC.D', 'ARCH_X86_32', append=True )
     elif build.match( 'x86_64-*' ):
         doc.add( 'LIBHB.GCC.D', 'ARCH_X86_64', append=True )
+    elif build.match( 'amd64-*' ):
+        doc.add( 'LIBHB.GCC.D', 'ARCH_X86_64', append=True )
 
     if options.enable_asm and ( not Tools.yasm.fail or options.enable_local_yasm ):
         asm = ''
@@ -1940,7 +1950,7 @@ int main()
             else:
                 doc.add( 'LIBHB.YASM.f', 'elf32' )
             doc.add( 'LIBHB.YASM.m', 'x86' )
-        elif build.match( 'x86_64-*' ):
+        elif build.match( 'x86_64-*' ) or build.match( 'amd64-*' ):
             asm = 'x86'
             doc.add( 'LIBHB.GCC.D', 'HAVE_MMX ARCH_X86_64', append=True )
             if build.match( '*-*-darwin*' ):
